/*
 * simuler un contrle dans une chane de production
 *
 * @author: harold trannois @ u-picardie
 * @author: M.Lokbani (adaptation ift1176 et validation jdk 1.3)
 *
 * Troisime Partie
 * ----------------
 *
 * Maintenant, nous allons faire en sorte que chaque sac soit vrifi par
 * un et un seul vrificateur.
 *
 * Pour cela, chaque vrificateur teste le Sac.
 *
 * Celui-ci mmorise son tat de vrification par une variable boolenne.
 *
 * Ainsi, chaque vrificateur teste la variable du Sac.
 *
 * Si celle-ci a la valeur fausse, il contrle le Sac, sinon, il passe au
 * suivant.
 *
 * Cette solution est... fausse telle qu'exprime.
 *
 * En effet, on a encore une fois un problme potentiel d'entrelacement :
 *
 *     ahmed teste le sac 0, il constate qu'il doit le vrifier 
 *     albert teste le sac 0, il constate qu'il doit le vrifier 
 *     ahmed commence la vrification 
 *     albert commence la vrification 
 *
 * Comme d'habitude, on va confier la responsabilit du contrle  un objet.
 *
 * Cette fois-ci, nous avons un objet tout trouver pour assumer cette
 * charge : le Sac lui-mme.
 *
 */


/*
 * Dfinir la classe ControleurSac qui implmente l'interface
 * Runnable.
 */


class ControleurSac implements Runnable {
    
    public ControleurSac(String inNom) {
	nom = inNom;
    }
    
    public void run() {
	for (int i=0 ; i<10 ; i++) {
	    Simulation3.lesSacs[i].tuEsVerifiePar(this);
	}
    }
    public String getNom() {
	return nom;
    }    
    String nom;
    
} 




class Sac {

    /*
     * ajouter une variable boolenne dejaVerifie dans la classe Sac. 
     */

    private int numero;
    private boolean dejaVerifie;
    

    public Sac(int inNumero) {
	numero = inNumero;
	dejaVerifie = false; // inutile, c'est la valeur par dfaut
    }
    
    /*
     * dans la mthode tuEsVerifiePar(ControleurSac), ajouter le test
     * de la variable et faire sortir immdiatement de la mthode si
     * cette variable est vraie
     */

    public synchronized void tuEsVerifiePar(ControleurSac inControleur) {
	if (dejaVerifie)
	    return;
	dejaVerifie = true;
	System.out.println(inControleur.getNom() + " commence le sac " + numero);
	try {
	    double delai = Math.random()*1000;
	    Thread.sleep( (int) delai);
	}
	catch(InterruptedException e) {
	}
	System.out.println(inControleur.getNom() + " a fini le sac " + numero);
    }
} 


    
public class Simulation3 {
    
    public final static int NB_SACS = 10;
    
    public static void main(String[ ] args) {
	lesSacs = new Sac[NB_SACS];
	for (int i=0 ; i < NB_SACS ; i++) {
	    lesSacs[i] = new Sac(i);
	}
	ControleurSac a;
	Thread t;
	a = new ControleurSac("Ahmed");
	t = new Thread(a);
	t.start();
	a = new ControleurSac("Albert");
	t = new Thread(a);
	t.start();
    }
    
    public static Sac[ ] lesSacs;
    
} 



/*
Ahmed commence le sac 0
Ahmed a fini le sac 0
Ahmed commence le sac 1
Ahmed a fini le sac 1
Ahmed commence le sac 2
Ahmed a fini le sac 2
Ahmed commence le sac 3
Ahmed a fini le sac 3
Ahmed commence le sac 4
Ahmed a fini le sac 4
Ahmed commence le sac 5
Ahmed a fini le sac 5
Ahmed commence le sac 6
Ahmed a fini le sac 6
Ahmed commence le sac 7
Ahmed a fini le sac 7
Ahmed commence le sac 8
Ahmed a fini le sac 8
Ahmed commence le sac 9
Ahmed a fini le sac 9

*/

